package dk.nsp.idsas.integrations.sores;

import java.io.IOException;

import dk.nsp.idsas.integrations.ServiceErrorCallback;

import dk.nsp.idsas.util.ErrorMessages;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import static dk.nsp.idsas.integrations.util.TestUtil.createHttpResponse;
import static org.junit.Assert.*;

public class SoresClientTest {
    private HttpClient httpClient;
    private final String url = "test-url";
    private final String validResponseJson = "{   \n" +
            "    \"\": [{\"sorId\": 343451000016007}],\n" +
            "    \"AdditionalRecordsExist\": true\n" +
            "}";
    private ServiceErrorCallback serviceErrorCallback;
    private SoresClient soresClient;

    @Before
    public void setUp() {
        httpClient = Mockito.mock(HttpClient.class);
        serviceErrorCallback = Mockito.mock(ServiceErrorCallback.class);
        soresClient = new SoresClient(httpClient, url, serviceErrorCallback);
    }

    @Test
    public void sores_WhenServiceReturnsValidResponse_ReturnsTrueAndRegistersServiceSuccess() throws IOException, SoresClientException {
        HttpResponse httpResponse = createHttpResponse(200, "application/json", validResponseJson);
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));
        String cvr = "1234";

        boolean cvrMapsToSor = soresClient.cvrMapsToSor(cvr);

        assertTrue(cvrMapsToSor);
        String expectedPath = url;
        Mockito.verify(httpClient).execute(
                Mockito.argThat(request -> expectedPath.equals(request.getURI().getPath())));
        Mockito.verify(serviceErrorCallback).registerServiceSuccess();
    }

    @Test
    public void sores_WhenClientThrowsIOException_ThrowsSoresClientExceptionAndRegisterError() throws IOException {
        Mockito.doThrow(IOException.class).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.cvrMapsToSor("1234"));

        assertEquals(ErrorMessages.soresError(), e.getMessage());
        Mockito.verify(serviceErrorCallback).registerServiceError();
    }

    @Test
    public void sores_WhenServiceReturnsResponse200WithInvalidJson_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse)
                .when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.cvrMapsToSor("1234"));

        assertEquals(ErrorMessages.soresError(), e.getMessage());
        Mockito.verify(serviceErrorCallback).registerServiceError();
    }

    @Test
    public void sores_WhenServiceReturnsResponse200WithoutCvrEntries_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "{\"foobar\": \"42\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.cvrMapsToSor("1234"));

        assertEquals(ErrorMessages.soresResponseError(), e.getMessage());
        Mockito.verify(serviceErrorCallback).registerServiceError();
    }

    @Test
    public void sores_WhenServiceReturnsResponse404WithInvalidJson_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(404, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.cvrMapsToSor("1234"));

        assertEquals(ErrorMessages.soresError(), e.getMessage());
        Mockito.verify(serviceErrorCallback).registerServiceError();
    }

    @Test
    public void sores_WhenServiceReturnsResponse404WithValidJson_ReturnsFalseAndRegistersServiceSuccess() throws IOException, SoresClientException {
        HttpResponse httpResponse = createHttpResponse(404, "application/json", validResponseJson);
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        boolean cvrMapsToSor = soresClient.cvrMapsToSor("1234");

        assertFalse(cvrMapsToSor);
        Mockito.verify(serviceErrorCallback).registerServiceSuccess();
    }

    @Test
    public void sores_WhenServiceReturnsResponse400_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(400, "application/json", "{\"test\": \"test\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.cvrMapsToSor("1234"));

        assertEquals(ErrorMessages.soresError(), e.getMessage());
        Mockito.verify(serviceErrorCallback).registerServiceError();
    }

}