package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class LookupServiceTest {

    private final String allowedClientSystemName = "TestSystem";

    private LookupService lookupService;
    private BlurringDao blurringDao;
    private AuditLogger auditLogger;

    @Before
    public void setUp() {
        blurringDao = Mockito.mock(BlurringDao.class);
        auditLogger = Mockito.mock(AuditLogger.class);
        lookupService = new LookupService(blurringDao, auditLogger, allowedClientSystemName);
    }

    @Test
    public void getActiveBlurrings_WithSundhedsFaglig_ThrowsException() {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        IdsasActor sundhedsFaglig = IdsasActor.createSundhedsFaglig("5678");

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> lookupService.getActiveBlurrings(sundhedsFaglig, patientId, patientIdType));

        // Assert
        assertEquals(ErrorMessages.userTypeNotSupported(IdsasActor.IdsasActorType.SUNDHEDSFAGLIG.name()), e.getMessage());
    }

    @Test()
    public void getActiveBlurrings_WithSystemUser_ReturnsActiveBlurringsAndAuditLogs() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        IdsasActor systemBruger = IdsasActor.createSystem("5678", allowedClientSystemName, "");
        String blurredOrganisation = "testOrg";
        Mockito.when(blurringDao.getActiveBlurrings(patientId, patientIdType))
                .thenReturn(Collections.singletonList(new BlurringEntity(null, null, blurredOrganisation, null, null)));

        // Act
        List<String> activeBlurrings = lookupService.getActiveBlurrings(systemBruger, patientId, patientIdType);

        // Assert
        assertEquals(1, activeBlurrings.size());
        assertEquals(blurredOrganisation, activeBlurrings.get(0));
        Mockito.verify(auditLogger).logGetBlurredOrganisations(patientId);
    }

    @Test()
    public void getActiveBlurrings_WithSystemUserWithoutAllowedClientSystemName_ThrowsException() {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        IdsasActor systemBrugerNoAccess = IdsasActor.createSystem("5678", "notAllowedClientSystemName", "");

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> lookupService.getActiveBlurrings(systemBrugerNoAccess, patientId, patientIdType));

        // Assert
        assertEquals(ErrorMessages.clientSystemNameError(systemBrugerNoAccess.getClientSystemName(), allowedClientSystemName), e.getMessage());
    }
}
