package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.security.SecureRandom;

import static org.junit.Assert.*;
import static org.mockito.ArgumentMatchers.any;

public class SaltServiceTest {

    private SaltDao saltDao;
    private WhitelistDao whitelistDao;
    private SaltService saltService;
    private SecureRandom secureRandom;

    @Before
    public void setUp() {
        saltDao = Mockito.mock(SaltDao.class);
        whitelistDao = Mockito.mock(WhitelistDao.class);
        secureRandom = Mockito.mock(SecureRandom.class);
        saltService = new SaltService(saltDao, whitelistDao, secureRandom);
    }

    @Test
    public void getCurrentSalt_WithNonSystemUser_ThrowsException() {
        IdsasActor sundhedsFagligActor = IdsasActor.createSundhedsFaglig("");

        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> saltService.getCurrentSalt(sundhedsFagligActor));

        assertEquals(ErrorMessages.userTypeNotSupported(sundhedsFagligActor.getActorType().name()), e.getMessage());
    }

    @Test
    public void getCurrentSalt_WhereClientKeyIsNotWhitelisted_ThrowsException() {
        IdsasActor systemActor = IdsasActor.createSystem("", "", "testClientKey");
        Mockito.when(whitelistDao.isWhitelisted(systemActor.getClientKey(), WhitelistType.SALT)).thenReturn(false);

        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> saltService.getCurrentSalt(systemActor));

        assertEquals(ErrorMessages.clientKeyWhitelistError(systemActor.getClientKey(), WhitelistType.SALT), e.getMessage());
    }

    @Test
    public void getCurrentSalt_WithWhitelistedSystemActor_ReturnsSalt() throws IdsasServiceException {
        IdsasActor systemActor = IdsasActor.createSystem("", "", "testClientKey");
        Mockito.when(whitelistDao.isWhitelisted(systemActor.getClientKey(), WhitelistType.SALT)).thenReturn(true);
        byte[] testSalt = {1};
        Mockito.when(saltDao.getLatestSalt()).thenReturn(testSalt);

        byte[] currentSalt = saltService.getCurrentSalt(systemActor);

        assertArrayEquals(testSalt, currentSalt);
    }

    @Test
    public void renewSalt_insertsSecureRandomBytesAsSalt() {
        byte[] newSalt = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
        Mockito.doAnswer(func -> {
            byte[] bytes = func.getArgument(0);
            System.arraycopy(newSalt, 0, bytes, 0, newSalt.length);
            return null;
        } ).when(secureRandom).nextBytes(any());

        saltService.renewSalt();

        Mockito.verify(saltDao).insert(newSalt);
    }
}